@@ -1,4 +1,4 @@
-# Copyright (c) 2019 - 2023, Adrian Dusa
+# Copyright (c) 2019 - 2024, Adrian Dusa
 # All rights reserved.
 # 
 # Redistribution and use in source and binary forms, with or without
@@ -9,13 +9,14 @@
 #     * Redistributions in binary form must reproduce the above copyright
 #       notice, this list of conditions and the following disclaimer in the
 #       documentation and/or other materials provided with the distribution.
-#     * The names of its contributors may NOT be used to endorse or promote products
-#       derived from this software without specific prior written permission.
+#     * The names of its contributors may NOT be used to endorse or promote
+#       products derived from this software without specific prior written
+#       permission.
 # 
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
-# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-# DISCLAIMED. IN NO EVENT SHALL ADRIAN DUSA BE LIABLE FOR ANY
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+# ARE DISCLAIMED. IN NO EVENT SHALL ADRIAN DUSA BE LIABLE FOR ANY
 # DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 # (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 # LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
@@ -25,8 +26,8 @@
 
 `possibleNumeric` <- function(x, each = FALSE) {
     result <- rep(NA, length(x))
-    isna <- is.na(x)
-    if (all(isna)) {
+    nax <- is.na(x)
+    if (all(nax)) {
         if (each) {
             return(result)
         }
@@ -35,7 +36,7 @@
     if (is.logical(x)) {
         if (each) {
             result <- logical(length(x))
-            result[isna] <- NA
+            result[nax] <- NA
             return(result)
         }
         return(FALSE)
@@ -50,7 +51,7 @@
     }
     if (is.numeric(x)) {
         if (each) {
-            result[!isna] <- TRUE
+            result[!nax] <- TRUE
             return(result)
         }
         return(TRUE)
@@ -58,17 +59,25 @@
     if (is.factor(x)) {
         x <- as.character(x)
     }
-    x <- gsub("\u00a0", " ", x) 
+    x <- gsub(
+        "\u00a0", 
+        " ",
+        gsub(
+            "\u009d", 
+            "",
+            x
+        )
+    )
     multibyte <- grepl("[^!-~ ]", x)
     if (any(multibyte)) {
-        isna[multibyte] <- TRUE
         result[multibyte] <- FALSE
-        x[multibyte] <- NA
     }
-    if (each) {
-        x <- suppressWarnings(as.numeric(na.omit(x)))
-        result[!isna] <- !is.na(x)
+    if (sum(nax) < length(x)) {
+        eachx <- suppressWarnings(as.numeric(x[!nax & !multibyte]))
+        result[!nax & !multibyte] <- !is.na(eachx)
+    }
+    if (each | length(x) == 1) {
         return(result)
     }
-    return(!any(is.na(suppressWarnings(as.numeric(na.omit(x))))))
+    return(all(result[!nax]))
 }
