@@ -1,4 +1,4 @@
-# Copyright (c) 2019 - 2023, Adrian Dusa
+# Copyright (c) 2019 - 2024, Adrian Dusa
 # All rights reserved.
 # 
 # Redistribution and use in source and binary forms, with or without
@@ -9,13 +9,14 @@
 #     * Redistributions in binary form must reproduce the above copyright
 #       notice, this list of conditions and the following disclaimer in the
 #       documentation and/or other materials provided with the distribution.
-#     * The names of its contributors may NOT be used to endorse or promote products
-#       derived from this software without specific prior written permission.
+#     * The names of its contributors may NOT be used to endorse or promote
+#       products derived from this software without specific prior written
+#       permission.
 # 
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
-# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-# DISCLAIMED. IN NO EVENT SHALL ADRIAN DUSA BE LIABLE FOR ANY
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
+# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+# ARE DISCLAIMED. IN NO EVENT SHALL ADRIAN DUSA BE LIABLE FOR ANY
 # DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 # (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 # LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
@@ -27,6 +28,14 @@
     UseMethod("using")
 }
 `using.default` <- function(data, expr, ...) {
+    if (missing(expr)) {
+        args <- unlist(lapply(match.call(), deparse)[-1])
+        args <- args[setdiff(names(args), c("data", "expr"))]
+        if (length(args) > 1) {
+            stopError("Missing or ambiguous expression")
+        }
+        expr <- str2lang(paste(names(args), args[[1]], sep = "<-"))
+    }
     eval(substitute(expr), data, enclos = parent.frame())
 }
 `using.data.frame` <- function(data, expr, split.by = NULL, ...) {
@@ -36,15 +45,25 @@
     split.by <- substitute(split.by)
     sby <- all.vars(split.by)
     nsby <- all.names(split.by)
+    if (missing(expr)) {
+        args <- unlist(lapply(match.call(), deparse)[-1])
+        args <- args[setdiff(names(args), c("data", "expr", "split.by"))]
+        if (length(args) > 1) {
+            stopError("Missing or ambiguous expression")
+        }
+        expr <- str2lang(paste(names(args), args[[1]], sep = "<-"))
+    }
     expr <- substitute(expr)
     vexpr <- all.vars(expr)
-    vexpr <- vexpr[is.element(vexpr, names(data))]
     if (any(vexpr == ".")) {
         vexpr <- colnames(data)
+    } else {
+        vexpr <- vexpr[is.element(vexpr, colnames(data))]
     }
     if (length(sby) == 0) {
         return(eval(expr = expr, envir = data, enclos = parent.frame()))
     }
+    cexpr <- as.character(expr)
     csby <- setdiff(as.character(split.by), c("c", "+", "&"))
     test <- unlist(lapply(seq(length(csby)), function(i) {
         tryCatchWEM(eval(parse(text = csby[i]), envir = data, enclos = parent.frame()))
@@ -158,10 +177,10 @@
     any_w_table <- any(
         sapply(res, function(x) class(x)[1] == "w_table")
     )
-    if (all(unlist(lapply(res, is.atomic))) & !any_w_table) {
+    if (all(sapply(res, is.atomic)) & !any_w_table) {
         classes <- unique(unlist(lapply(res, class)))
         classes <- setdiff(classes, c("integer", "double", "character", "numeric", "complex"))
-        lengths <- unlist(lapply(res, length))
+        lengths <- sapply(res, length)
         result <- matrix(NA, nrow = length(res), ncol = max(lengths))
         for (i in seq(length(res))) {
             if (!is.null(res[[i]])) {
@@ -170,11 +189,30 @@
         }
         result[] <- coerceMode(round(result, 3))
         rownames(result) <- apply(slexp, 1, function(x) paste(x, collapse = ", "))
+        expr <- as.list(expr)
         if (max(lengths) == 1) {
-            colnames(result) <- as.character(as.list(expr)[[1]])
+            colnames(result) <- as.character(expr[[1]])
         }
         else {
-            colnames(result) <- names(res[[which.max(lengths)]])
+            expr <- sapply(expr, function(x) as.character(as.list(x))[[1]])
+            nms <- names(res[[which.max(lengths)]])
+            if (is.null(nms)) {
+                if (as.character(expr) == "c" && max(lengths) == length(expr) - 1) {
+                    nms <- expr[-1]
+                }
+                else {
+                    nms <- rep(" ", max(lengths))
+                }
+            }
+            if (
+                any(nms == "") &&
+                expr[1] == "c" &&
+                is.element("summary", expr) &&
+                sum(nms == "") == length(expr) - 2
+            ) {
+                nms[nms == ""] <- setdiff(expr, c("c", "summary"))
+            }
+            colnames(result) <- nms
         }
         res <- result
         class(res) <- c("admisc_fobject", "matrix")
